@@ -1,9 +1,9 @@
 package App::wordlist;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-05-21'; # DATE
+our $DATE = '2020-05-22'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.271'; # VERSION
+our $VERSION = '0.272'; # VERSION
 
 use 5.010001;
 use strict;
@@ -117,7 +117,7 @@ _
         action => {
             schema  => ['str*', in=>[
                 'list_cpan', 'list_installed',
-                'grep',
+                'grep', 'stat',
             ]],
             default => 'grep',
             cmdline_aliases => {
@@ -131,6 +131,11 @@ _
                     is_flag => 1,
                     code => sub { my $args=shift; $args->{action} = 'list_cpan' },
                 },
+                s => {
+                    summary=>'Show statistics contained in the wordlist modules',
+                    is_flag => 1,
+                    code => sub { my $args=shift; $args->{action} = 'stat' },
+                },
             },
         },
         lcpan => {
@@ -263,7 +268,7 @@ sub wordlist {
     my $use_color = ($color eq 'always' ? 1 : $color eq 'never' ? 0 : undef)
         // $ENV{COLOR} // (-t STDOUT);
 
-    if ($action eq 'grep') {
+    if ($action eq 'grep' || $action eq 'stat') {
         # convert /.../ in arg to regex
         for (@$arg) {
             $_ = Encode::decode('UTF-8', $_);
@@ -287,8 +292,45 @@ sub wordlist {
                 push @$wordlists, $rec->{name};
             }
         }
+
         $wordlists = [shuffle @$wordlists] if $random;
-        log_trace "Wordlists to use: %s", $wordlists;
+        log_trace "Wordlist(s) to use: %s", $wordlists;
+
+        if ($action eq 'stat') {
+            no strict 'refs';
+            return [200] unless @$wordlists;
+            my %all_stats;
+            for my $wl (@$wordlists) {
+                my $mod = "WordList::$wl";
+                (my $modpm = "$mod.pm") =~ s!::!/!g;
+                require $modpm;
+                if (@$wordlists == 1) {
+                    return [200, "OK", \%{"$mod\::STATS"}];
+                } else {
+                    $all_stats{$wl} = \%{"$mod\::STATS"};
+                }
+            }
+            return [200, "OK", \%all_stats];
+        }
+
+        # optimize: use the wordlist module's pick() if we only use a single
+        # wordlist
+        if ($random && @$wordlists == 1 && $num > 0 && $num <= 100) {
+            no strict 'refs';
+            my $mod = "WordList::$wordlists->[0]";
+            (my $modpm = "$mod.pm") =~ s!::!/!g;
+            require $modpm;
+            my $stats = \%{"$mod\::STATS"};
+            my $wl;
+            if ($stats && $stats->{num_words} > 50_000) {
+                require WordListMod;
+                $wl = WordListMod::get_mod_wordlist(
+                    $wordlists->[0], "RandomSeekPick");
+            } else {
+                $wl = $mod->new;
+            }
+            return [200, "OK", [$wl->pick($num)]];
+        }
 
         my $n = 0;
 
@@ -319,7 +361,10 @@ sub wordlist {
                 my $mod = "WordList::$wl";
                 (my $modpm = "$mod.pm") =~ s!::!/!g;
                 log_trace "Loading wordlist module $mod ...";
-                eval { require $modpm; $wl_obj = $mod->new };
+                eval {
+                    require $modpm;
+                    $wl_obj = $mod->new;
+                };
                 if ($@) {
                     warn;
                     $i_wordlist++;
@@ -384,6 +429,8 @@ sub wordlist {
             }
             $res = [200, "OK", \@words];
         }
+
+      RETURN_RES:
         $res;
 
     } elsif ($action eq 'list_installed') {
@@ -464,7 +511,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.271 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-21.
+This document describes version 0.272 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-22.
 
 =head1 SYNOPSIS
 
